"""Client-side models for the Nomic Platform API."""

from enum import Enum

from pydantic import BaseModel, Field

__all__ = [
    "ContentExtractionMode",
    "OcrLanguage",
    "TableSummaryOptions",
    "FigureSummaryOptions",
    "ParseOptions",
    "ExtractOptions",
    "ParseRequest",
    "ExtractRequest",
]


class ContentExtractionMode(str, Enum):
    """The overall strategy for extracting content from the document."""

    Metadata = "metadata"  # Disable all OCR. Only use embedded document text.
    Hybrid = "hybrid"  # Use a VLM for tables, and run an OCR model on all bitmaps found in the document.
    Ocr = "ocr"  # Use a VLM for tables. Run an OCR model on full pages.


class OcrLanguage(str, Enum):
    """Language selection for OCR."""

    English = "en"
    Latin = "latin"
    Chinese_Japanese_English = "zh_ja_en"


class TableSummaryOptions(BaseModel):
    """Options for generating table summaries."""

    enabled: bool = Field(
        default=False,
        description="Whether to generate a summary of table content",
    )


class FigureSummaryOptions(BaseModel):
    """Options for generating figure summaries."""

    enabled: bool = Field(
        default=True,
        description="Whether to generate a summary of figure content",
    )


class ParseOptions(BaseModel):
    """Options to customize document parsing."""

    content_extraction_mode: ContentExtractionMode = Field(
        default=ContentExtractionMode.Hybrid,
        description="The overall strategy for extracting content from the document",
    )
    ocr_language: OcrLanguage = Field(
        default=OcrLanguage.English,
        description="Language selection for OCR",
    )
    table_summary: TableSummaryOptions | None = Field(
        default=None,
        description="Options for generating table summaries",
    )
    figure_summary: FigureSummaryOptions | None = Field(
        default=None,
        description="Options for generating figure summaries",
    )


class ExtractOptions(BaseModel):
    """Options to customize document extraction."""

    system_prompt: str | None = Field(
        default=None,
        description="Custom system prompt to guide the AI extraction process across the entire file. "
        "Use this to provide specific instructions, context, or constraints for how information "
        "should be extracted and formatted according to your requirements.",
    )


class ParseRequest(BaseModel):
    """Request model for parsing a document."""

    file_url: str = Field(description="File URL to process")
    options: ParseOptions = Field(
        default_factory=ParseOptions,
        description="Options to customize document parsing",
    )


class ExtractRequest(BaseModel):
    """Request model for extracting data from documents."""

    file_urls: list[str] = Field(description="List of file URLs to process for extraction")
    extraction_schema: dict = Field(description="JSON schema defining the structure of data to extract")
    system_prompt: str | None = Field(
        default=None,
        description="Custom system prompt to guide the AI extraction process",
    )



import base64
import gc
import random
import sys
from io import BytesIO
from pathlib import Path
from typing import Optional, Union
from uuid import UUID

import pyarrow as pa
import requests
from pyarrow import ipc

nouns = [
    "newton",
    "einstein",
    "gauss",
    "pascal",
    "laplace",
    "euler",
    "kepler",
    "wiles",
    "neumann",
    "noether",
    "bohr",
    "heisenberg",
    "planck",
    "maxwell",
    "hawking",
    "feynman",
    "curie",
    "poincare",
    "turing",
    "nash",
    "hilbert",
    "godel",
    "galois",
    "ramanujan",
    "lovelace",
    "hypatia",
    "leibniz",
    "copernicus",
    "hubble",
    "darwin",
    "hooke",
    "faraday",
    "pasteur",
    "mendel",
    "galvani",
    "bell",
    "boltzmann",
    "dirac",
    "fermi",
    "oppenheimer",
    "schrodinger",
    "fibonacci",
    "chatelet",
    "herschel",
    "kovalevskaya",
    "somerville",
    "franklin",
    "lepore",
    "yalow",
    "jemison",
    "burnell",
    "bartik",
    "spencer",
    "hamilton",
    "johnson",
    "ride",
    "anning",
    "meitner",
    "hopper",
    "mcclintock",
    "franklin",
    "noether",
    "rubin",
    "wu",
    "fermat",
    "cantor",
    "jordan",
    "riemann",
    "babbage",
    "dijkstra",
    "shannon",
    "erdos",
    "boole",
    "lagrange",
    "lebesgue",
    "legendre",
    "gauss",
    "cauchy",
    "ramanujan",
    "weierstrass",
    "fourier",
    "cavendish",
    "lavoisier",
    "becquerel",
    "avogadro",
    "dalton",
    "rutherford",
    "thomson",
    "bose",
    "fisher",
    "jaynes",
    "bernoulli",
    "alembert",
    "gibbs",
    "pareto",
    "levy",
    "sierpinski",
    "hardy",
    "wigner",
    "mercator",
    "ortelius",
    "speed",
    "schedel",
    "waldseemuller",
    "homann",
    "rennell",
    "ogilby",
    "hondius",
    "moll",
    "blaeu",
    "sanson",
    "borgonio",
    "isle",
    "munster",
    "hollar",
    "kitchin",
    "lecun",
    "bengio",
    "hinton",
    "schmidhuber",
    "fei-fei",
    "koller",
    "russell",
    "norvig",
    "vapnik",
    "avram",
    "bishop",
    "goodfellow",
    "kingma",
    "arora",
    "sutton",
    "breiman",
]

adjectives = [
    "curious",
    "innovative",
    "analytical",
    "creative",
    "logical",
    "adventurous",
    "inquisitive",
    "imaginative",
    "dynamic",
    "quirky",
    "resourceful",
    "persistent",
    "observant",
    "insightful",
    "intuitive",
    "experimental",
    "inspiring",
    "inventive",
    "tenacious",
    "stubborn",
    "lazy",
    "disorganized",
    "inattentive",
    "careless",
    "unpredictable",
    "irrational",
    "impulsive",
    "chaotic",
    "indecisive",
    "cynical",
    "neglectful",
    "overbearing",
    "oblivious",
    "inflexible",
    "lackadaisical",
    "truculent",
    "diffident",
    "lumbering",
]


def arrow_iterator(table: pa.Table):
    # TODO: setting to 10k so it doesn't take too long?
    # Wrote this as a generator so we don't realize the whole table in memory
    reader = table.to_reader(max_chunksize=10_000)
    for batch in reader:
        for item in batch.to_pylist():
            yield item


def b64int(i: int):
    ibytes = int.to_bytes(i, length=8, byteorder="big").lstrip(b"\x00")
    if ibytes == b"":
        ibytes = b"\x00"
    return base64.b64encode(ibytes).decode("utf8").rstrip("=")


def get_random_name():
    return f"{random.choice(adjectives)}-{random.choice(nouns)}"


def assert_valid_project_id(project_id):
    try:
        uuid_obj = UUID(project_id, version=4)
    except ValueError:
        raise ValueError(f"`{project_id}` is not a valid project id.")


def get_object_size_in_bytes(obj):
    marked = {id(obj)}
    obj_q = [obj]
    sz = 0

    while obj_q:
        sz += sum(map(sys.getsizeof, obj_q))

        # Lookup all the object referred to by the object in obj_q.
        # See: https://docs.python.org/3.7/library/gc.html#gc.get_referents
        all_refr = ((id(o), o) for o in gc.get_referents(*obj_q))

        # Filter object that are already marked.
        # Using dict notation will prevent repeated objects.
        new_refr = {o_id: o for o_id, o in all_refr if o_id not in marked and not isinstance(o, type)}

        # The new obj_q will be the ones that were not marked,
        # and we will update marked with their ids so we will
        # not traverse them again.
        obj_q = new_refr.values()
        marked.update(new_refr.keys())

    return sz


# Helpful function for downloading feather files
# Best for small feather files
def download_feather(
    url: Union[str, Path], path: Path, headers: Optional[dict] = None, num_attempts=1, overwrite=False
) -> pa.Schema:
    """
    Download a feather file from a URL to a local path.
    Returns the schema of feather file if successful.

    Parameters:
        url (str): URL to download feather file from.
        path (Path): Local path to save feather file to.
        headers (dict): Optional headers to include in request.
        num_attempts (int): Number of download attempts before raising an error.
        overwrite (bool): Whether to overwrite existing file.
    Returns:
        Feather schema.
    """
    assert num_attempts > 0, "Num attempts must be greater than 0"
    download_attempt = 0
    download_success = False
    schema = None
    while download_attempt < num_attempts and not download_success:
        download_attempt += 1
        if not path.exists() or overwrite:
            # Attempt download
            try:
                data = requests.get(str(url), headers=headers)
                readable = BytesIO(data.content)
                readable.seek(0)
                tb = pa.feather.read_table(readable, memory_map=False)  # type: ignore
                schema = tb.schema
                path.parent.mkdir(parents=True, exist_ok=True)
                pa.feather.write_feather(tb, path)  # type: ignore
                download_success = True
            except pa.ArrowInvalid:
                # failed try again
                path.unlink(missing_ok=True)
        else:
            # Load existing file
            try:
                schema = ipc.open_file(path).schema
                download_success = True
            except pa.ArrowInvalid:
                path.unlink(missing_ok=True)
    if not download_success or schema is None:
        raise ValueError(f"Failed to download feather file from {url} after {num_attempts} attempts.")
    return schema



import json
import os
import time
from collections.abc import Sequence
from dataclasses import dataclass
from enum import Enum, auto
from pathlib import Path
from typing import Any, Generic, Literal, TypeVar, overload
from urllib.parse import urlparse

import jsonschema
import requests

from nomic.dataset import AtlasClass

from .client_models import (
    ContentExtractionMode,
    ExtractOptions,
    ExtractRequest,
    FigureSummaryOptions,
    OcrLanguage,
    ParseOptions,
    ParseRequest,
    TableSummaryOptions,
)

__all__ = [
    "NomicClient",
    "PlatformTask",
    "TaskFailed",
    "TaskPending",
    "UploadedFile",
    # Client models
    "ContentExtractionMode",
    "ExtractOptions",
    "FigureSummaryOptions",
    "OcrLanguage",
    "ParseOptions",
    "TableSummaryOptions",
]

MAX_FAILRESP_LENGTH = 1_000  # chars

T = TypeVar("T")

client: "AtlasClass | None" = None


def get_client():
    global client
    if client is None:
        client = AtlasClass()
    return client


@dataclass(frozen=True)
class UploadedFile:
    url: str


class Sentinel(Enum):
    Nothing = auto()


class TaskPending(Exception):
    pass


class TaskFailed(Exception):
    pass


class PlatformTask(Generic[T]):
    """
    An object representing a task on the Nomic Platform.

    Attributes:
        id: The ID of the task.
    """

    _id: str
    _result: "T | Sentinel"

    def __init__(self, id: str):
        self._id = id
        self._result = Sentinel.Nothing

    @property
    def id(self) -> str:
        return self._id

    def get(self, timeout: "float | None" = None, *, block: bool = True) -> T:
        """
        Waits for the task to complete and returns the result.

        Args:
            timeout: The maximum time to wait for the task to complete.
            block: Whether to block until the task is complete.

        Returns:
            The result of the task.

        Raises:
            TaskPending: If the task is not complete and block is True.
            TaskFailed: If the task fails.
        """
        if self._result is not Sentinel.Nothing:
            return self._result
        client = get_client()
        start_time = time.time()
        while True:
            response = client._get(f"/v1/status/{self._id}")
            raise_for_status_with_body(response)
            status_resp = response.json()
            if status_resp["status"] == "COMPLETED":
                break
            if status_resp["status"] == "FAILED":
                raise TaskFailed(status_resp["error"])
            if not block:
                raise TaskPending
            sleeptime = 1  # poll interval
            if timeout is not None:
                end_time = start_time + timeout
                if end_time < (now := time.time()):
                    raise TaskPending
                sleeptime = min(sleeptime, end_time - now)
            time.sleep(sleeptime)

        completed_response = requests.get(status_resp["result_url"])
        raise_for_status_with_body(completed_response)

        result = status_resp.pop("result", {})
        result.pop("result_url", None)
        result.pop("result", None)
        result["result"] = completed_response.json()
        result["result"].pop("status", None)
        result["result"].pop("error", None)
        self._result = result
        return result


class NomicClient:
    """Client for the Nomic Platform API."""

    def upload_file(self, path: "str | os.PathLike[str]") -> UploadedFile:
        """
        Uploads a file to the Nomic Platform.

        Args:
            path: The path to the PDF file to upload.

        Returns:
            An UploadedFile object representing the uploaded file.
        """
        client = get_client()

        path = Path(path)

        with path.open("rb") as pdf_file:
            file_type = path.suffix.lower()
            if file_type == ".pdf":
                content_type = "application/pdf"
            # elif file_type == ".docx":
            #   content_type = "application/vnd.openxmlformats-officedocument.wordprocessingml.document"
            # elif file_type == ".doc":
            #   content_type = "application/msword"
            # elif file_type == ".txt":
            #   content_type = "text/plain"
            # elif file_type == ".pptx":
            #   content_type = "application/vnd.openxmlformats-officedocument.presentationml.presentation"
            # elif file_type == ".ppt":
            #   content_type = "application/vnd.ms-powerpoint"
            # elif file_type == ".csv":
            #   content_type = "text/csv"
            # elif file_type == ".xlsx":
            #   content_type = "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"
            # elif file_type == ".xls":
            #   content_type = "application/vnd.ms-excel"
            else:
                raise ValueError(f"Unsupported file type: {file_type}")

            response = client._post(
                "/v1/upload",
                json=dict(files=[{"id": path.name, "size": path.stat().st_size, "content_type": content_type}]),
            )
            raise_for_status_with_body(response)

            values = response.json()

            # Extract from the files array
            file_info = values["files"][0]
            upload_url = file_info["upload_url"]
            nomic_url = file_info["nomic_url"]

            # upload the file to the designated pre-signed url
            resp = requests.put(upload_url, data=pdf_file, headers={"x-amz-server-side-encryption": "AES256"})

        raise_for_status_with_body(resp)
        return UploadedFile(url=nomic_url)

    @overload
    def parse(
        self, file: "str | UploadedFile", *, options: "ParseOptions | None" = ..., block: Literal[True] = ...
    ) -> "dict[str, Any]": ...
    @overload
    def parse(
        self, file: "str | UploadedFile", *, options: "ParseOptions | None" = ..., block: Literal[False]
    ) -> PlatformTask["dict[str, Any]"]: ...
    @overload
    def parse(self, file: "str | UploadedFile", *, options: "ParseOptions | None" = ..., block: bool) -> Any: ...

    def parse(self, file: "str | UploadedFile", *, options: "ParseOptions | None" = None, block: bool = True) -> Any:
        """
        Parses a document into a structured JSON representation.

        Args:
            file: The file to parse. Can be a string URL or an UploadedFile object.
            options: Optional ParseOptions to customize parsing behavior (OCR settings, etc.).
            block: Whether to block until the task is complete.

        Returns:
            By default, returns the parsed document. If block is False, returns a PlatformTask that can be used to get
            the result.

        Raises:
            ValueError: If an invalid URL is passed.
            TaskFailed: If block is True and the task fails.

        Example:
            Complete end-to-end workflow with upload and parsing:

            ```python
            from nomic.client import NomicClient

            # Upload a PDF file
            client = NomicClient()
            file = client.upload_file("my_document.pdf")

            # Parse the document
            result = client.parse(file)
            print(result)
            ```
        """
        client = get_client()

        file_url = self._file_to_url(file)
        if options is not None:
            request = ParseRequest(file_url=file_url, options=options)
        else:
            request = ParseRequest(file_url=file_url)
        payload = request.model_dump(mode="json", exclude_unset=True)

        response = client._post("/v1/parse", json=payload)
        raise_for_status_with_body(response)
        task = PlatformTask(response.json()["task_id"])
        if block:
            return task.get()
        return task

    @overload
    def extract(
        self,
        files: "str | UploadedFile | Sequence[str | UploadedFile]",
        schema: "dict[str, Any]",
        *,
        options: "ExtractOptions | None" = ...,
        block: Literal[True] = ...,
    ) -> Any: ...
    @overload
    def extract(
        self,
        files: "str | UploadedFile | Sequence[str | UploadedFile]",
        schema: "dict[str, Any]",
        *,
        options: "ExtractOptions | None" = ...,
        block: Literal[False],
    ) -> PlatformTask[Any]: ...
    @overload
    def extract(
        self,
        files: "str | UploadedFile | Sequence[str | UploadedFile]",
        schema: "dict[str, Any]",
        *,
        options: "ExtractOptions | None" = ...,
        block: bool,
    ) -> Any: ...

    def extract(
        self,
        files: "str | UploadedFile | Sequence[str | UploadedFile]",
        schema: "dict[str, Any]",
        *,
        options: "ExtractOptions | None" = None,
        block: bool = True,
    ) -> Any:
        """
        Extracts structured data from documents.

        Args:
            files: List of uploaded files to extract from.
            schema: A JSON schema defining the structure of data to extract.
            options: Optional ExtractOptions to customize extraction behavior (system prompt, etc.).
            block: Whether to block until the task is complete.

        Returns:
            By default, returns the extracted data matching the provided schema. If block is False, returns a PlatformTask
            that can be used to get the result.

        Raises:
            ValueError: If an invalid URL is passed.
            TaskFailed: If block is True and the task fails.

        Example:
            Complete end-to-end workflow with upload and extraction:

            ```python
            from nomic.client import NomicClient

            # Upload a PDF file
            client = NomicClient()
            file = client.upload_file("my_document.pdf")

            # Define extraction schema
            schema = {
                "type": "array",
                "items": {
                    "type": "object",
                    "properties": {
                        "speaker": {"type": "string"},
                        "content": {"type": "string"},
                    }
                },
            }

            # Extract structured data
            result = client.extract(file, schema)
            print(result)
            ```
        """
        jsonschema.Draft7Validator.check_schema(schema)

        if isinstance(files, (str, UploadedFile)):
            files = [files]

        client = get_client()

        request = ExtractRequest(
            file_urls=list(map(self._file_to_url, files)),
            extraction_schema=schema,
            system_prompt=options.system_prompt if options is not None else None,
        )
        payload = request.model_dump(mode="json", exclude_unset=True)

        response = client._post("/v1/extract", json=payload)
        raise_for_status_with_body(response)
        task = PlatformTask(response.json()["task_id"])
        if block:
            return task.get()
        return task

    @staticmethod
    def _file_to_url(file: "str | UploadedFile") -> str:
        if isinstance(file, UploadedFile):
            return file.url
        parsed = urlparse(file)
        if parsed.scheme in ("nomic", "http", "https"):
            return file
        if parsed.scheme == "file" or (not parsed.scheme and Path(file).exists()):
            raise ValueError(
                f"Cannot directly pass local file to platform: {file!r}\nPlease use upload_file() to upload it first."
            )
        if not parsed.scheme:
            raise ValueError(f"Invalid URL: {file!r}")
        raise ValueError(f"Unsupported scheme {parsed.scheme!r} for URL {file!r}")


def raise_for_status_with_body(resp: requests.Response) -> None:
    """
    Raises HTTPError if the response is not successful.

    Like Response.raise_for_status, but includes the (truncated) response body in the
    exception message for improved diagnostics.

    Args:
        resp: The response to check

    Raises:
        requests.HTTPError: If the response is not successful
    """
    http_error_msg = ""
    if isinstance(resp.reason, bytes):
        # We attempt to decode utf-8 first because some servers
        # choose to localize their reason strings. If the string
        # isn't utf-8, we fall back to iso-8859-1 for all other
        # encodings. (See PR #3538)
        try:
            reason = resp.reason.decode("utf-8")
        except UnicodeDecodeError:
            reason = resp.reason.decode("iso-8859-1")
    else:
        reason = resp.reason

    if 400 <= resp.status_code < 500:
        http_error_msg = f"{resp.status_code} Client Error: {reason} for url: {resp.url}"

    elif 500 <= resp.status_code < 600:
        http_error_msg = f"{resp.status_code} Server Error: {reason} for url: {resp.url}"

    if http_error_msg:
        if (ctype := resp.headers.get("content-type")) is not None:
            http_error_msg += f"\nContent-Type: {ctype}"
        http_error_msg += f"\nBody: {format_body(resp)}"
        raise requests.HTTPError(http_error_msg, response=resp)


def format_body(resp: requests.Response) -> str:
    text = None
    if (ctype := resp.headers.get("content-type")) and "application/json" in ctype.lower():
        try:
            data = resp.json()
            text = json.dumps(data, indent=2, ensure_ascii=False)
        except Exception:
            pass
    if text is None:
        text = resp.text

    limit = MAX_FAILRESP_LENGTH
    if len(text) > limit:
        return text[:limit] + f"\n… [truncated {len(text) - limit} chars]"
    return text



import json
import os
import time
from pathlib import Path

import click
import jwt
import requests
from rich.console import Console

tenants = {
    "staging": {"frontend_domain": "staging-atlas.nomic.ai", "api_domain": "staging-api-atlas.nomic.ai"},
    "production": {"frontend_domain": "atlas.nomic.ai", "api_domain": "api-atlas.nomic.ai"},
}

nomic_base_path = Path.home() / ".nomic"


def validate_api_http_response(response):
    if response.status_code >= 500 and response.status_code < 600:
        raise Exception("Cannot contact establish a connection with Nomic services.")

    return response


def get_api_credentials(fn=None):
    if fn is None:
        fn = "credentials"
    filepath = nomic_base_path / fn
    if not filepath.exists():
        raise ValueError("You have not configured your Nomic API token. Run `nomic login` to configure.")

    with open(filepath, "r") as file:
        credentials = json.load(file)
        return credentials


def login(token, tenant="production", domain=None):
    if tenant == "enterprise" and domain is None:
        raise ValueError("Enterprise tenants must specify their deployment domain.")

    if domain is not None:
        tenants["enterprise"] = {"frontend_domain": domain, "api_domain": f"api.{domain}"}

    if tenant not in tenants:
        raise ValueError("Invalid tenant.")
    environment = tenants[tenant]
    auth0_auth_endpoint = f"https://{environment['frontend_domain']}/cli-login"

    console = Console()
    style = "bold"
    if not token:
        console.print("Authenticate with the Nomic API", style=style, justify="center")
        console.print(auth0_auth_endpoint, style=style, justify="center")
        console.print(
            "Click the above link to retrieve your access token and then run `nomic login \\[token]`",
            style=style,
            justify="center",
        )
        exit()

    # save credential
    if not nomic_base_path.exists():
        nomic_base_path.mkdir()

    expires = None
    refresh_token = None

    if token.startswith("nk-"):
        bearer_token = token
    else:
        refresh_token = token
        response = requests.get("https://" + environment["api_domain"] + f"/v1/user/token/refresh/{token}")
        response = validate_api_http_response(response)

        if not response.status_code == 200:
            raise Exception("Could not authorize you with Nomic. Run `nomic login` to re-authenticate.")
        bearer_token = response.json()["access_token"]
        decoded_token = jwt.decode(bearer_token, options={"verify_signature": False})
        expires = decoded_token["exp"]

    with open(os.path.join(nomic_base_path, "credentials"), "w") as file:
        saved_credentials = {
            "refresh_token": refresh_token,
            "token": bearer_token,
            "tenant": tenant,
            "expires": expires,
        }

        if tenant == "enterprise":
            saved_credentials = {**saved_credentials, **environment}
        json.dump(saved_credentials, file)


def refresh_bearer_token():
    credentials = get_api_credentials()
    if credentials["expires"] and time.time() >= credentials["expires"]:
        try:
            environment = tenants[credentials["tenant"]]
        except KeyError:
            environment = credentials
        response = requests.get(
            "https://" + environment["api_domain"] + f"/v1/user/token/refresh/{credentials['refresh_token']}"
        )
        response = validate_api_http_response(response)

        if not response.status_code == 200:
            raise Exception("Could not authorize you with Nomic. Run `nomic login` to re-authenticate.")

        bearer_token = response.json()["access_token"]
        credentials["token"] = bearer_token
        with open(os.path.join(nomic_base_path, "credentials"), "w") as file:
            json.dump(credentials, file)
    return credentials


def switch(tenant):
    assert tenant in ["staging", "production", None]
    credentials = get_api_credentials()
    current_tenant = credentials["tenant"]
    if tenant is None:
        print(f"Current tenant: {current_tenant}")
        return
    if current_tenant == tenant:
        return
    else:
        current_loc = nomic_base_path / "credentials"
        new_loc = nomic_base_path / f"credentials_{current_tenant}"
        print(f"Switching from {current_tenant} to {tenant}.")
        if current_loc.exists():
            current_loc.rename(new_loc)
        if (nomic_base_path / f"credentials_{tenant}").exists():
            (nomic_base_path / f"credentials_{tenant}").rename(current_loc)
        else:
            login(token=None, tenant=tenant)


@click.command()
@click.option("--domain", default=None, help="Domain to use for Atlas enterprise login")
@click.argument("command", nargs=1, default="")
@click.argument("params", nargs=-1)
def cli(command, params, domain=None):
    if command == "login":
        if len(params) == 0:
            login(token=None, tenant="production")
        if len(params) == 1 and params[0] == "staging":
            login(token=None, tenant="staging")
        if len(params) == 2 and params[0] == "staging":
            login(token=params[1], tenant="staging")
        if len(params) == 1 and params[0] == "enterprise":
            if domain is None:
                raise ValueError("Must pass --domain to log into an enterprise environment")
            login(token=None, tenant="enterprise", domain=domain)
        if len(params) == 2 and params[0] == "enterprise":
            if domain is None:
                raise ValueError("Must pass --domain to log into an enterprise environment")
            login(token=params[1], tenant="enterprise", domain=domain)
        if len(params) == 1:
            login(token=params[0], tenant="production")
    elif command == "switch":
        if len(params) == 0:
            switch(tenant=None)
        if len(params) == 1:
            switch(tenant=params[0])
    else:
        raise ValueError(f"Command {command} not found.")


if __name__ == "__main__":
    cli()



"""Setup file for the Atlas Client"""

import os
import sys
from setuptools import setup, find_packages

description = "The official Nomic python client."

# Read README.md and remove tables and images
with open("README.md") as f:
    content = f.read()
    # Remove table sections including content
    while "<table>" in content and "</table>" in content:
        start = content.find("<table>")
        end = content.find("</table>") + 8
        content = content[:start] + content[end:]
    # Remove img tags and content
    while "<img" in content and ">" in content:
        start = content.find("<img")
        end = content.find(">", start) + 1
        content = content[:start] + content[end:]
    long_description = content

setup(
    name="nomic",
    version="3.8.0",
    url="https://github.com/nomic-ai/nomic",
    description=description,
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(include=["nomic", "nomic.*"]),
    author_email="support@nomic.ai",
    author="nomic.ai",
    classifiers=[
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3",
    ],
    requires_python='>=3.10',
    install_requires=[
        "click",
        "jsonlines",
        "jsonschema>=4.23.0,<5",
        "loguru",
        "rich",
        "requests",
        "numpy",
        "pandas",
        "pydantic>=2,<3",
        "tqdm",
        "pyarrow",
        "pillow",
        "pyjwt",
    ],
    extras_require={
        "local": [
            "gpt4all>=2.5.0,<3",
        ],
        "aws": ["boto3", "sagemaker"],
        "all": [
            "nomic[local,aws]",
        ],
        "dev": [
            "nomic[all]",
            "black==24.3.0",
            "coverage",
            "pylint",
            "pytest",
            "isort",
            "pyright<=1.1.377",
            "myst-parser",
            "mkdocs-material",
            "mkautodoc",
            "twine",
            "mkdocstrings[python]",
            "mkdocs-jupyter",
            "pillow",
            "cairosvg",
            "pytorch-lightning",
            "pandas",
        ],
    },
    entry_points={
        "console_scripts": ["nomic=nomic.cli:cli"],
    },
    include_package_data=True,
)



def _check_aws_dependencies():
    try:
        import boto3
    except ImportError as e:
        missing_package = str(e).split("No module named ")[-1].strip("'")
        raise ImportError(
            f"The '{missing_package}' package is required for this feature. "
            "Please install it by running 'pip install nomic[aws]'."
        )


_check_aws_dependencies()

from .sagemaker import embed_image, embed_text
